#ifndef PROGRAMACIONJABATA_H
#define PROGRAMACIONJABATA_H

//#include <util/PlatformUtils.hpp>
//#include <sax2/XMLReaderFactory.hpp>
//
#include <iostream>
#include <sax2/SAX2XMLReader.hpp>

#include "jabatoBlock.h"
#include "jabatoBlockHandler.hpp"
#include "jabatoBlocksHandler.hpp"
#include "jabatoScheduleHandler.hpp"

#include <qstring.h>

#define cout     std::cout
#define cerr     std::cerr
#define endl	 std::endl

class JabatoSchedule {
public:
  JabatoSchedule();
  ~JabatoSchedule();

  bool initSchedule(SAX2XMLReader *parser=NULL, bool loadSounds=true);

  const char *getStationName() const {return stationName;};

  const bool autoRecMode() const {return autoRecording;}
  const uint samplingFreq() const {return samplingFrequency;}

  uint blocksCount();
  const int getBlockId(uint i);
  const char *getBlockName(uint i);
  const char *getBlockURL(uint i);

  int getBlockPosition(int dow, int hh, int mm);

  void checkTasks(int,int,int);

  JabatoStrip *getCurrentStrip();

  JabatoBlock * getDefaultBlock();
  JabatoBlock * getBlockAt(int index);

private:
//  JabatoBlock currentBlock;
//  JabatoBlock defaultBlock;

  QList<JabatoBlock> blocks;

  JabatoStrip currentStrip;


  char *stationName;
  bool autoRecording;
  uint samplingFrequency;

  QString *jSFile;
  QString *jBFile;

  SAX2XMLReader         *parser;
  jabatoBlockHandler    *blockH;
  jabatoBlocksHandler   *blocksH;
  jabatoScheduleHandler *scheduleH;

  void findJabatoFiles();
  bool initParser(SAX2XMLReader *parser=NULL);
  bool externalParser;
  bool loadSounds;
};

// ---------------------------------------------------------------------------
//  This is a simple class that lets us do easy (though not terribly efficient)
//  trancoding of XMLCh data to local code page for display.
// ---------------------------------------------------------------------------
class StrX
{
public :
    // -----------------------------------------------------------------------
    //  Constructors and Destructor
    // -----------------------------------------------------------------------
    StrX(const XMLCh* const toTranscode)
    {
        // Call the private transcoding method
        fLocalForm = XMLString::transcode(toTranscode);
    }

    ~StrX()
    {
        delete [] fLocalForm;
    }

    // -----------------------------------------------------------------------
    //  Getter methods
    // -----------------------------------------------------------------------
    const char* localForm() const
    {
        return fLocalForm;
    }

private :
    // -----------------------------------------------------------------------
    //  Private data members
    //
    //  fLocalForm
    //      This is the local code page form of the string.
    // -----------------------------------------------------------------------
    char*   fLocalForm;
};

//inline ostream& 
inline std::ostream& operator<< 
               (std::ostream& target, const StrX& toDump)
{
    target << toDump.localForm();
    return target;
}

#endif
