#include <sax2/Attributes.hpp>
#include <sax2/DefaultHandler.hpp>

class JabatoTask {
public:
  JabatoTask();
  ~JabatoTask();

  int dow, hour, minute;
  char *shellCmd;

  void checkTask(int d, int h, int m);
private:
  bool waiting;

};


class jabatoScheduleHandler : public DefaultHandler
{
public:

    jabatoScheduleHandler();
    ~jabatoScheduleHandler();

    void findStationName();
    void find(int dow, int hh, int mm, JabatoStrip *);
		void loadScheduledTasks();

    bool found();
    bool error();

    char *getStationName();
    bool autoRecMode();
    uint samplingFreq();
    void checkScheduledTasks(int d, int h, int m);

    // -----------------------------------------------------------------------
    //  Implementations of the SAX DocumentHandler interface
    // -----------------------------------------------------------------------
    //    void characters(const XMLCh* const chars, const unsigned int length);
    void endElement( const XMLCh* const uri,
					 const XMLCh* const localname,
					 const XMLCh* const qname);

    void startElement(	const   XMLCh* const    uri,
			const   XMLCh* const    localname,
			const   XMLCh* const    qname,
		        const   Attributes&		attributes);

    void startDocument();

				
    // -----------------------------------------------------------------------
    //  Implementations of the SAX ErrorHandler interface
    // -----------------------------------------------------------------------
    void warning(const SAXParseException& exception);
    void error(const SAXParseException& exception);
    void fatalError(const SAXParseException& exception);

private :
  enum tolookfor {
    JABATONAME,
    JABATOSTRIP,
    SCHEDULEDTASKS,
    PROGRAMSTRIPS
  };

  int	lookingFor;
	
  int	day;
  uint	moment;
  uint  begin;
  uint  end;
  JabatoStrip *jabatoStrip;
  char	*stationName;
  bool   autoRecording;
  uint   samplingFrequency;
  bool 	 m_error;
  bool 	 m_found;
  bool   readingDay;

  uint   beginHour, beginMinute, endHour, endMinute;

  QList<JabatoTask> tasks;
	
		
  // You may know how to make this constants static class const.
  const XMLCh *SCHEDULE;
  const XMLCh *STATIONNAME;
  const XMLCh *AUTORECORDING;
  const XMLCh *SAMPLINGFREQUENCY;


  const XMLCh *DAY;
  const XMLCh *DAYOFTHEWEEK;

  const XMLCh *TIMESTRIP;
  const XMLCh *BEGINATHOUR;
  const XMLCh *BEGINATMINUTE;
  const XMLCh *ENDATHOUR;
  const XMLCh *ENDATMINUTE;
  const XMLCh *BLOCKID;
  const XMLCh *RECORDINGSTRIP;

  const XMLCh *EXECAT;
  const XMLCh *HOUR;
  const XMLCh *MINUTE;
  const XMLCh *SHELLCMD;
};
