
#include <qtimer.h>
#include <qdir.h>

#include "jabatoSound.moc"

#include "level.h"
#include "fft.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

#define MAXSONGSPERSECOND 16

#define MINPROGRAMSECS  60
#define MINSLOTSECS 	3

#define MAXSILENTSECS		60
#define MAXVALIDSILENTSECS	3

char *mode_names[5] =
{"stereo", "j-stereo", "dual-ch", "single-ch", "multi-ch"};
char *layer_names[3] =
{"I", "II", "III"};
char *version_names[3] =
{"MPEG-1", "MPEG-2 LSF", "MPEG-2.5"};
char *version_nums[3] =
{"1", "2", "2.5"};


unsigned int bitrates[3][3][15] =
{
 {
  {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448},
  {0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384},
  {0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320}
 },
 {
  {0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256},
  {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160},
  {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160}
 },
 {
  {0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256},
  {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160},
  {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160}
 }
};


unsigned int s_freq[3][4] =
{
    {44100, 48000, 32000, 0},
    {22050, 24000, 16000, 0},
    {11025, 8000, 8000, 0}
};

char *Layer::mode_name()
{
    return (mode_names[mode]);
}

char *Layer::layer_name()
{
    return (layer_names[lay - 1]);
}

char *Layer::version_name()
{
    return (version_names[version]);
}

char *Layer::version_num()
{
    return (version_nums[version]);
}

unsigned int Layer::bitrate()
{
    return (bitrates[version][lay - 1][bitrate_index]);
}

unsigned int Layer::sfreq()
{
    return (s_freq[version][sampling_frequency]);
}

bool Layer::get(char * fname)
{
    FILE *file;
    struct stat fi;
    unsigned char buff[1024];
    unsigned char *buffer;
    size_t filelen;
    size_t temp;
    size_t readsize;

   	file = fopen(fname, "r");
    if (file != NULL) {
			fstat(fileno(file), &fi);
			filelen = fi.st_size;
    } else {
			return (false);
    }

    /* Theoretically reading 1024 instead of just 4 means a performance hit
     * if we transfer over net filesystems... However, no filesystem I know
     * of uses block sizes under 1024 bytes.
     */
    fseek(file, 0, SEEK_SET);
    readsize = fread(&buff, 1, 1024, file);
    fclose(file);
    readsize -= 4;
    if (readsize <= 0)
      return (false);

    buffer = buff-1;

    /* Scan through the block looking for a header */
    do {
      buffer++;
      temp = ((buffer[0] << 4) & 0xFF0) | ((buffer[1] >> 4) & 0xE);
    } while ((temp != 0xFFE) && ((size_t)(buffer-buff)<readsize));

    if (temp != 0xFFE) {
	return (false);
    } else {
	switch ((buffer[1] >> 3 & 0x3)) {
	case 3:
	    version = 0;
	    break;
	case 2:
	    version = 1;
	    break;
	case 0:
	    version = 2;
	    break;
	default:
	    return (false);
	}
	lay = 4 - ((buffer[1] >> 1) & 0x3);
	error_protection = !(buffer[1] & 0x1);
	bitrate_index = (buffer[2] >> 4) & 0x0F;
	sampling_frequency = (buffer[2] >> 2) & 0x3;
	padding = (buffer[2] >> 1) & 0x01;
	extension = buffer[2] & 0x01;
	mode = (buffer[3] >> 6) & 0x3;
	mode_ext = (buffer[3] >> 4) & 0x03;
	copyright = (buffer[3] >> 3) & 0x01;
	original = (buffer[3] >> 2) & 0x1;
	emphasis = (buffer[3]) & 0x3;
	stereo = (mode == 3) ? 1 : 2;
	
        if (bitrates[version][lay - 1][bitrate_index])
          seconds = filelen / (bitrates[version][lay - 1][bitrate_index] * 125);
        else
          return (false);

        return (true);
    }
}


/* ---------------------------------------------------------------------- */
JabatoSound::JabatoSound() {
    mfilepath = mrecordpath = mname = mdir = msubdir = mpath = "";
    toLanDiskCmd = toWanDiskCmd = "";
    totalsecs = mhours = mminutes = mseconds = 0;
    soundType = NOTDEFINED;
    started = false;
}

void
JabatoSound::clear() {
    mfilepath = mrecordpath = mname = mdir = msubdir = mpath = "";
    toLanDiskCmd = toWanDiskCmd = "";
    totalsecs = mhours = mminutes = mseconds = 0;
    soundType = NOTDEFINED;
    started = false;
}

bool
JabatoSound::hasSound() {
  return (soundType != NOTDEFINED);
}

bool
JabatoSound::isProgram() {
  return soundType==JABATOPROGRAM;
}

bool
JabatoSound::isSong(){
  return soundType==JABATOSONG;
}

bool
JabatoSound::isSlot(){
  return soundType==JABATOSLOT;
}

bool
JabatoSound::isText(){
  return soundType==JABATOTEXT; 
}

bool
JabatoSound::isOGG(){
  return mext.contains("ogg",false)==1;
}

bool
JabatoSound::isMP3(){
  return mext.contains("mp3",false)==1;
}

bool
JabatoSound::isURL(){
  return mext.contains("http",true)==1;
}

bool
JabatoSound::isFESTIVAL(){
  return mext.contains("festival",false)==1;
}


void
JabatoSound::setSoundType(int i){
  soundType = i;
  updateTime();
}

void
JabatoSound::setFullPath(const char *fullpath){
  
  mfilepath = fullpath;

  mrecordpath="";
  mname = mdir = msubdir = mpath = "";
  totalsecs = mhours = mminutes = mseconds = 0;
  toLanDiskCmd = toWanDiskCmd = "";

  if(mfilepath.length()>4) {
    int from, to;

    file.setFile(fullpath);

    if(mfilepath.contains("http://",false))
      mext="http";
    else
      mext = file.extension(false);

    to = mfilepath.findRev(".",mfilepath.length());
    if(to<0) return;
    from = mfilepath.findRev("/",to);
    mname = mfilepath.mid(from+1,to-from-1);

    to = from-1;
    if(to<0) return;
    from = mfilepath.findRev("/",to-1);
    if(from<0) return;
    mdir = mfilepath.mid(from+1,to-from);

    to = from-1;
    if(to<0) return;
    from = mfilepath.findRev("/",to-1);
    if(from<0) return;
    msubdir = mfilepath.mid(from+1,to-from);

    to = from;
    if(to<0) return;
    mpath = mfilepath.mid(0,to);
  }
  //DEBUG(fprintf(stderr, "jabatoSound::setFullPath() %s/%s/%s\n",msubdir.data(),mdir.data(),mname.data()));
}


bool
JabatoSound::fileExists() {
  return (file.exists() && (file.size()>100));
}


void
JabatoSound::fileDate(int *y, int *m, int *d) {
  if(file.exists()) {
    file.refresh();
    QDate date = file.lastModified().date();
    *y=date.year();
    *m=date.month();
    *d=date.day();
  }
  else
    *y = *m = *d = 0;
}


void
JabatoSound::updateTime()
{
  char *filename;
  int  filesize;
  if(mrecordpath.length()>0) {
    filename = (char *)mrecordpath.data();
    filesize = (int)qfile.size();
  }
  else {
    filename = (char *)mfilepath.data();
    filesize = file.size();
  }
	  
  if(qfile.exists() || 
     (file.exists() && mrecordpath.length()==0)) {
    int secs = -1;
    if(isOGG()) 
      secs=totalsecs=filesize/13485;
    else if(isFESTIVAL()) 
      secs=totalsecs=filesize/13;
    else if(isMP3()) {
      if(!mp3info.get(filename) ||
       (soundType==JABATOSONG)&&(mp3info.bitrate()<128))  //vbr?
        secs = totalsecs = filesize/18000;
      else
        secs = totalsecs = mp3info.seconds;
    }
    else 
      secs = totalsecs = 0;
    mhours = secs / 3600;
    secs = secs - (mhours * 3600);
    mminutes = secs / 60;
    secs = secs - (mminutes * 60);
    mseconds = secs;
    //DEBUG(fprintf(stderr, "jabatoSound::updateTime() %s.mp3: %d:%02d:%02d\n",mname.data(),mhours,mminutes,mseconds));
  }
  else {
   //DEBUG(fprintf(stderr, "jabatoSound::updateTime() %s.mp3 NO EXISTE\n",mname.data()));
    totalsecs = mhours = mminutes = mseconds = 0;
  }
}


const char *
JabatoSound::getFullPath()  const {
  return mfilepath.data();
}


const char *
JabatoSound::getRecordPath() const {
  return mrecordpath.data();
}

const char *
JabatoSound::getName() const {
  return mname.data();
}

const char *
JabatoSound::getDirectory() const {
  return mdir.data();
}

const char *
JabatoSound::getSubdirectory() const {
  return msubdir.data();
}

const char *
JabatoSound::getPathToSub() const {
  return mpath.data();
}

int
JabatoSound::getHours() const {
  return mhours;
}

int
JabatoSound::getMinutes() const {
  return mminutes;
}

int
JabatoSound::getSeconds() const {
  return mseconds;
}

int
JabatoSound::getTimeInSeconds() const {
  return totalsecs;
}

void
JabatoSound::setTimeInSeconds(int secs){
  totalsecs = secs;
  mhours = secs / 3600;
  secs -= mhours * 3600;
  mminutes = secs / 60;
  secs -= mminutes * 60;
  mseconds = secs;
}

void
JabatoSound::startTimer() {
  elapsed.start();
  started = true;
}

void
JabatoSound::stopTimer() {
  elapsed.start();
  started = false;
}

bool
JabatoSound::timerStarted() {
  return started;
}

void
JabatoSound::setRecordingDate() {
  int day, month, year;
  static char newpath[512];
  char num_sec[3]="";
  int i_num_sec=1;
  
  recordedOn = QDateTime::currentDateTime().date();

  year=recordedOn.year();
  month=recordedOn.month();
  day=recordedOn.day();
  do {
    if(this->isProgram()) 
      sprintf(newpath,
         "%s/%04d%02d%02d%s-%s.%s",
         RECORDERPROGRAMSDIR,
         year, month, day, num_sec,
         mname.data(), mext.data());
    else
      sprintf(newpath,
         "%s/%04d%02d%02d%s-%s_%s.%s",
         RECORDERSLOTSDIR,
         year, month, day, num_sec,
         mdir.data(),
         mname.data(),mext.data());
    qfile.setName(newpath);
    i_num_sec++;
    sprintf(num_sec,"%02d",i_num_sec);
  } while (qfile.exists());
  mrecordpath=newpath;
}

void
JabatoSound::getRecordedOn(int *y, int *m, int *d) {
  *y=recordedOn.year();
  *m=recordedOn.month();
  *d=recordedOn.day();
}

void
JabatoSound::getElapsedTime(int *h, int *m, int *s) {
  int i = elapsed.elapsed()/1000;
  *h = i/3600;
  *m = (i/60) % 60;
  *s = i % 60;
}

void
JabatoSound::getLeftTime(int *h, int *m, int *s) {
  int i = totalsecs - (elapsed.elapsed()/1000);
  if(i>0) {
    *h = i/3600;
    *m = (i/60) % 60;
    *s = i % 60;
  }
  else {
    *h = 0;
    *m = 0;
    *s = 0;
  }
}

int
JabatoSound::getElapsedTimeInSeconds() {
  return elapsed.elapsed()/1000;
}

int
JabatoSound::getLeftTimeInSeconds() {
  int i = (totalsecs - (elapsed.elapsed()/1000));
  return (i>0?i:0);
}


void
JabatoSound::setToLanDiskCmd(char *cmd) {
  toLanDiskCmd = cmd;
}

void
JabatoSound::setToWanDiskCmd(char *cmd) {
  toWanDiskCmd = cmd;
}

bool
JabatoSound::execBackupCommands() {
 static char name[512];
 static char path[512];
 static char recpath[512];
	
 qfile.setName(mrecordpath);
 updateTime();
 
 if((soundType==JABATOPROGRAM && totalsecs<MINPROGRAMSECS) ||
    (soundType==JABATOSLOT && totalsecs<MINSLOTSECS))
 {
   fprintf(stderr, "jabatoSound::execBackupCommands: removing %s\n",
		   mrecordpath.data());
   qfile.remove();
   mrecordpath="";

   return false;
 }
 else {
   if((toLanDiskCmd.length()>0)||(toWanDiskCmd.length()>0)) 
   {
     fprintf(stderr, "jabatoSound::execBackupCommands: %s LAN:%s WAN:%s\n", 
		  mname.data(), toLanDiskCmd.data(), toWanDiskCmd.data());
     sprintf(name, "\"%s\"", mname.data());
     sprintf(path, "\"%s\"", mfilepath.data());
     sprintf(recpath, "\"%s\"", mrecordpath.data());
     if(toLanDiskCmd.length()>0) 
     {
        if(toLanDiskProcess.isRunning()) {
	    fprintf(stderr, 
              "jabatoSound::execBackupCommands: TOLANDISKCMD STILL RUNNING!\n");
	  	toLanDiskProcess.kill();
     }
  	toLanDiskProcess.clearArguments();
  	toLanDiskProcess << toLanDiskCmd.data() << name << path << recpath;
  	toLanDiskProcess.start(KProcess::DontCare, KProcess::NoCommunication);
   }
  if(toWanDiskCmd.length()>0) {
	if(toWanDiskProcess.isRunning()) {
	  fprintf(stderr, "jabatoSound::execBackupCommands: TOWANDISKCMD STILL RUNNING!\n");
	  	toWanDiskProcess.kill();
	}
	toWanDiskProcess.clearArguments();
  	toWanDiskProcess << toWanDiskCmd.data() << name << path << recpath;
  	toWanDiskProcess.start(KProcess::DontCare, KProcess::NoCommunication);
     }
   }
//0.9.5
//   mrecordpath="";
//   updateTime();
   return true;
 }
}

/* ---------------------------------------------------------------------- */
MP3Player::MP3Player(char *dev)
{
    player  = new KProcess();

    connect(player, SIGNAL(processExited (KProcess*)),
              this, SLOT(decoding_finished(KProcess*)));
    if(dev) {
      strcpy(devname,dev);
      sprintf(oggdevname,"dsp:%s",dev);
    }
    else
      devname[0]='\0';
    
}

MP3Player::~MP3Player()
{
    delete player;
}

bool
MP3Player::play(JabatoSound *js)
{
static int songs=-1;
static QTime elapsed;

  if(++songs>MAXSONGSPERSECOND) {
    if(elapsed.elapsed()<1000)
      return false;
    elapsed.start();
    songs = 1;
  }
  else if(songs==0) elapsed.start();


  jabatoSound = js;

  if (player->isRunning())
    player->kill();
  player->clearArguments();

  if(jabatoSound->isMP3() || jabatoSound->isURL()) {
    if(strlen(devname)>0) {
      if(!strcmp(devname,"esd") || !strcmp(devname,"arts")) {
       *player << "mpg123" << "-q";
       *player << "-o" << devname;
      }
      else {
        if(!strncmp(devname,"/dev/dsp",strlen("/dev/dsp"))) {
            *player << "mpg123" << "-q";
            *player << "-o"  << "oss" << "-a" << devname;  // Tarjeta Auxiliar
        }
        else {
            *player << "mpg123" << "-q";
            *player << "-o"  << "alsa09" << "-a" << devname;
        }
      }
    }
    *player << jabatoSound->getFullPath();
  }
  else if(jabatoSound->isFESTIVAL()) {
    *player << "festival" << "--tts";
    *player << jabatoSound->getFullPath();
  } else {
    bool addpath = true;
    if(strlen(devname)>0) {
      if(!strcmp(devname,"esd") || !strcmp(devname,"arts")) {
        *player << "ogg123" << "-q";
        *player << "-d" << devname;
      }
      else {
        if(!strncmp(devname,"/dev/dsp",strlen("/dev/dsp"))) {
            *player << "ogg123" << "-q";
            *player << "-d" << "oss" << "-o" << oggdevname; //"dsp:/dev/dsp3"; // Tarjeta Aux
        }
        else {
            *player << "sox" << "-q";
            *player << "-t"  << "ogg" << jabatoSound->getFullPath() << "-t"<< "alsa" << devname;
            addpath = false;
        }
      }
    }
    if (addpath)
      *player << jabatoSound->getFullPath();
  }

  fprintf(stderr, "PLAYING %s\n",jabatoSound->getFullPath());

  return(player->start(KProcess::NotifyOnExit, KProcess::Stdin));
}

bool
MP3Player::stop() {
//    return(player->kill(2));
    fprintf(stderr, "STOPING %s\n",jabatoSound->getFullPath());
    return(player->kill(9));
}

void
MP3Player::decoding_finished(KProcess *kproc) {
    DEBUG(fprintf(stderr, "MP3Player::decoding_finished()\n"));

    emit finished(this);
}

/* ---------------------------------------------------------------------- */

MP3Recorder::MP3Recorder(int rate, int channels, int tr, char *dev) {
  memset(&sndparams,0,sizeof(struct SOUNDPARAMS));
  levelWid = NULL;
  fftWid = NULL;
  recording = 0;
  encoderReading = 0;
  closeStdinPending = 0;
  inBuffer = 0;

  card = new Soundcard(dev);

  sndparams.rate     	= rate;
  sndparams.channels 	= channels;
  sndparams.format   		= FMT_16BIT;
  card->setparams(&sndparams);
  trigger	= tr;

  connect(card,SIGNAL(newparams(struct SOUNDPARAMS*)),
	        this, SLOT(new_params(struct SOUNDPARAMS*)));
  connect(card,SIGNAL(senddata(void*)),
          this, SLOT(new_data(void*)));

  encoder = new KProcess();

  connect(encoder, SIGNAL(wroteStdin(KProcess*)),
          this, SLOT(bufferjustread(KProcess*)));
  connect(encoder, SIGNAL(processExited (KProcess*)),
          this, SLOT(encoding_finished(KProcess*)));
}

MP3Recorder::~MP3Recorder() {
  delete(card);
  if(levelWid)
    delete(levelWid);
  if(fftWid)
    delete(fftWid);
}

void
MP3Recorder::setLevelFrame(QFrame *fr) {

  LevelWindow *levelwin = new LevelWindow(fr,"level");
  levelWid = levelwin;
  levelFrame = fr;

  levelwin->setMinimumSize(30,10);
  levelwin->PowervsMax = 0;
  levelwin->setGeometry(4,3,fr->width()-8,fr->height()-5);

  connect(card,SIGNAL(senddata(void*)),
          levelwin, SLOT(new_data(void*)));
  connect(card,SIGNAL(newparams(struct SOUNDPARAMS*)),
                   levelwin, SLOT(new_params(struct SOUNDPARAMS*)));

  card->setparams(&sndparams);

}


void
MP3Recorder::setFFTFrame(QFrame *fr) {

  FFTWindow *fftwin = new FFTWindow(fr,"level");
  fftWid = fftwin;
  fftFrame = fr;

  fftwin->setGeometry(2,2,fr->width()-4,fr->height()-4);

  connect(card,SIGNAL(senddata(void*)),
                   fftwin, SLOT(new_data(void*)));
  connect(card,SIGNAL(newparams(struct SOUNDPARAMS*)),
                   fftwin, SLOT(new_params(struct SOUNDPARAMS*)));

  card->setparams(&sndparams);
}


void
MP3Recorder::resizeFrames() {
  if(levelWid)
    levelWid->setGeometry(2,2,levelFrame->width()-3,levelFrame->height()-3);

  if(fftWid)
    fftWid->setGeometry(2,2,fftFrame->width()-3,fftFrame->height()-3);

}


void
MP3Recorder::setRecordParams(int rate, int channels, int tr) {
  sndparams.rate     	= rate;
  sndparams.channels 	= channels;
  trigger	= tr;
}

/*
void
MP3Recorder::renameSoundFile() {
  static char newfullpath[512];
  int day, month, year;
  QDir dir;

  jabatoSound->fileDate(&year,&month,&day);

  if(jabatoSound->isProgram()) 
    sprintf(newfullpath,
         "%s/programas/%04d%02d%02d-%s.mp3",
	 RECORDERBACKUPDIR,
         year, month, day,
         jabatoSound->getName());
  else
    sprintf(newfullpath,
         "%s/cunias/%04d%02d%02d-%s_%s.mp3", 
	 RECORDERBACKUPDIR,
         year, month, day,
         jabatoSound->getDirectory(),
         jabatoSound->getName());
  dir.rename(jabatoSound->getFullPath(),(const char *)&newfullpath);
}
*/

void
MP3Recorder::checkProgramDirectory() {
  QDir dir(jabatoSound->getPathToSub());

  if(dir.exists()) {
    if(!dir.cd(jabatoSound->getSubdirectory())) {
      fprintf(stderr,"ATENCIÓN: Creando carpeta %s para %s\n",
                    jabatoSound->getSubdirectory(),jabatoSound->getName());
      dir.mkdir(jabatoSound->getSubdirectory());
      if(!dir.cd(jabatoSound->getSubdirectory())) {
        fprintf(stderr,"ERROR CREANDO/ACCEDIENDO A %s\n",
                            jabatoSound->getSubdirectory());
        return;
      }
    }
    if(!dir.cd(jabatoSound->getDirectory())) {
      fprintf(stderr,"ATENCIÓN: Creando carpeta %s para %s\n",
                           jabatoSound->getDirectory(),jabatoSound->getName());
      dir.mkdir(jabatoSound->getDirectory());
      if(!dir.cd(jabatoSound->getDirectory())) {
        fprintf(stderr,"ERROR CREANDO/ACCEDIENDO A %s\n",
                            jabatoSound->getDirectory());
        return;
      }
    }
  }
}

bool
//MP3Recorder::start(JabatoSound *js, const char *albumtag=NULL, const char *artisttag=NULL) {
MP3Recorder::start(JabatoSound *js, const char *albumtag, const char *artisttag) {
  char tag[30];
  int day, month, year;

  DEBUG(fprintf(stderr,"MP3Recorder::start()"));

  jabatoSound = js;
  
  if(!jabatoSound->isOGG() && !jabatoSound->isMP3()) {
    fprintf(stderr,"MP3Recorder::start(): What's %s?\n",
		    jabatoSound->getFullPath());
    return false;
  }

//  if(jabatoSound->isProgram()) {
//  if(jabatoSound->fileExists())
//    renameSoundFile();
//  else 
    if(jabatoSound->isProgram()) 
      checkProgramDirectory();
//  }

  jabatoSound->setRecordingDate();
  jabatoSound->getRecordedOn(&year,&month,&day);

  if (encoder->isRunning())
    encoder->kill();

  wait      = 1;
  encoder->clearArguments();
  if(jabatoSound->isOGG()) {
    *encoder << "oggenc" << "-";
    *encoder << "-r";
    if(jabatoSound->isSlot())
      *encoder << "--title" << jabatoSound->getName();
    else
      *encoder << "--title" << jabatoSound->getDirectory();
    sprintf(tag, "%04d%02d%02d", year, month, day);
    *encoder << "--date" << tag;
    if(albumtag)
      *encoder << "--album" << albumtag;
    if(artisttag)
      *encoder << "--artist" << artisttag;

    if (sndparams.channels == 1)
      *encoder << "-C" << "1";
    else
      *encoder << "-C" << "2";

    sprintf(tag, "%d", sndparams.rate);
    *encoder << "-R" << tag;
    if(sndparams.rate>22050)
      *encoder << "-q" << "4";
    else
      *encoder << "-q" << "2";
    //  *encoder << "-b" << "64";
      
    //*encoder << "-o" << jabatoSound->getFullPath();
    *encoder << "-o" << jabatoSound->getRecordPath();
  }
  else {
    *encoder << "lame";
    *encoder << "-r" << "-x" << "-h";
    *encoder << "--tt" << jabatoSound->getDirectory();
    if(jabatoSound->isSlot())
      sprintf(tag, "Grabada el %d/%d/%04d", day, month, year);
    else
      sprintf(tag, "Grabado el %d/%d/%04d", day, month, year);
    *encoder << "--tc" << tag;
    sprintf(tag, "%04d", year);
    *encoder << "--ty" << tag;
    if(albumtag)
      *encoder << "--tl" << albumtag;
    if(artisttag)
      *encoder << "--ta" << artisttag;
    if (sndparams.channels == 1)
      *encoder << "-m" << "m";
    else
      *encoder << "-m" << "j";
    *encoder << "-s";
    switch (sndparams.rate) {
      case 16000:
        *encoder <<  "16"; //<< "-b" << "24";
        break;
      case 22050:
        *encoder << "22.05";
        break;
      case 24000:
        *encoder << "24";
        break;
      case 32000:
        *encoder << "32";
        break;
      case 44100:
        *encoder << "44.1";
        break;
      case 48000:
        *encoder << "48";
        break;
      default:
        fprintf(stderr,
		"MP3Recorder::start() unsupported sampling frequency!!\n");
        return false;
    }
    //*encoder << "-" << jabatoSound->getFullPath();
    *encoder << "-" << jabatoSound->getRecordPath();
  }

  DEBUG(fprintf(stderr,"%s\n",jabatoSound->getRecordPath()));
  recording = -1;
  if(0==card->start_record()) {
    return encoder->start(KProcess::NotifyOnExit, KProcess::Stdin);
  }
  else {
    fprintf(stderr,"MP3Recorder::start() cannot start recording!!\n");
    return false;
  }
}

/*
bool
MP3Recorder::start(JabatoSound *js, const char *albumtag=NULL, const char *artisttag=NULL) {
  char tag[30];
  int day, month, year;

  DEBUG(fprintf(stderr,"MP3Recorder::start()\n"));

  jabatoSound = js;

//  if(jabatoSound->isProgram()) {
    if(jabatoSound->fileExists())
      renameSoundFile();
    else 
      if(jabatoSound->isProgram()) 
        checkProgramDirectory();
//  }

  jabatoSound->setRecordingDate();
  jabatoSound->getRecordedOn(&year,&month,&day);

  if (encoder->isRunning())
    encoder->kill();

  wait      = 1;
  encoder->clearArguments();

  recording = -1;
  if(0==card->start_record()) {
    return encoder->start(KProcess::NotifyOnExit, KProcess::Stdin);
  }
  else {
    fprintf(stderr,"MP3Recorder::start() cannot start recording!!\n");
    return false;
  }
}
*/

void
MP3Recorder::finish() {
  DEBUG(fprintf(stderr,"MP3Recorder::finish()-->"));
  recording = 0;
  card->stop();
  if(encoderReading) {
    DEBUG(fprintf(stderr,"encoder still reading. Wait a little...\n"));
    closeStdinPending = -1;
    QTimer::singleShot(600, this, SLOT(check_if_finished()));
  }
  else {
    DEBUG(fprintf(stderr,"encoder free. Let's close it!!\n"));
    closeStdinPending = 0;
    waitingExitNotification = -1;
    encoder->closeStdin();
    QTimer::singleShot(300, this, SLOT(check_if_finished()));
  }
}


void
MP3Recorder::new_params(struct SOUNDPARAMS *p)
{
    DEBUG(fprintf(stderr,"MP3Recorder::new_params\n"));
    memcpy(&sndparams,p,sizeof(struct SOUNDPARAMS));
}

bool
MP3Recorder::minSignal(void *data){
  int             i,max;
  short          *s;
      
  for (i = (sndparams.blocksize>>1)-1, s=(short*)data, max = 0; i; i--)
    if (abs(s[i]) > max) max = abs(s[i]);
  
  max = max * 100 / 32768;
  return max >= trigger;
}

/* SLOTS */
void
MP3Recorder::new_data(void *data)
{
  static int silent_sec;
  static bool holded=false;
  
  if(recording) {
   	
    if (wait) {
      if(minSignal(data)) {
        wait = 0;
        emit started(this);
      }
      else
	return;
    }

    if (encoderReading) {
      if(inBuffer) {
        if(inBuffer+sndparams.blocksize>RECORDERBUFFERSIZE) 
          fprintf(stderr,"!");
	else {
	  long ptr=(long)&buffer;
	  ptr+=inBuffer;
          memcpy((void *)ptr,data,sndparams.blocksize);
 	  inBuffer+=sndparams.blocksize;
	}
      }
      else {
        fprintf(stderr,"<");
        inBuffer = sndparams.blocksize;
        memcpy(&buffer,data,inBuffer);
      }
    }
    else {
      if(minSignal(data)) { 
	if(silent_sec) {      
          silent_sec=0;
	  if(holded) {
            holded=false;
            emit restarted(this);
	  }
	}
      }
      else if(silent_sec) {
	int silent_secs = jabatoSound->getElapsedTimeInSeconds() - silent_sec;
	if(silent_secs > MAXSILENTSECS) {
	  holded=false;
    	  fprintf(stderr,
	        "\nMP3Recorder::new_data() silent_secs -> %d stoping...\n",
		silent_secs);
	  finish();
	  return;
	}
	else if(silent_secs > MAXVALIDSILENTSECS) { 
	  if(!holded) {
	    holded=true;
    	    emit hold(this);
	  }
          fprintf(stderr,"_");
	  return;
	}
      }
      else {
	silent_sec = jabatoSound->getElapsedTimeInSeconds();
	holded=false;
    	fprintf(stderr,
	        "MP3Recorder::new_data() silent_sec -> %d\n",silent_sec);
      }
	      
      encoderReading = -1;
      if(FALSE==encoder->writeStdin((char *)data, sndparams.blocksize)) {
    	fprintf(stderr,
	        "MP3Recorder::new_data() cannot write on encoder stdin\n");
    	finish();
      }
    }
  }
}


void
MP3Recorder::bufferjustread(KProcess *proc) {
  if(inBuffer) {
    if(FALSE==encoder->writeStdin((char *)&buffer, inBuffer)) {
      fprintf(stderr,
	      "MP3Recorder::new_data() cannot write on encoder stdin\n");
      finish();
    }
    fprintf(stderr,"%d/%d>",RECORDERBUFFERSIZE,inBuffer);
    inBuffer = 0;
  }
  else {
    encoderReading = 0;
    if (closeStdinPending)
    {
      fprintf(stderr,
          "MP3Recorder::bufferjustread() close encoder stdin pending. Closing it...\n");
      encoder->closeStdin();
      waitingExitNotification = -1;
      closeStdinPending = 0;
      QTimer::singleShot(300, this, SLOT(check_if_finished()));
    }
  }
}


void
MP3Recorder::encoding_finished(KProcess *) {
  DEBUG(fprintf(stderr,"MP3Recorder::encoding_finished()\n"));

  if(recording) {
    recording = 0;
    card->stop();
  }

  waitingExitNotification = 0;
  emit finished(this);
}

void
MP3Recorder::check_if_finished() {
  DEBUG(fprintf(stderr,"MP3Recorder::check_if_finished() "));

  if(closeStdinPending) {
    encoder->closeStdin();
    waitingExitNotification = -1;
    closeStdinPending = 0;
    QTimer::singleShot(300, this, SLOT(check_if_finished()));
    DEBUG(fprintf(stderr," closing STDIN first...\n"));
  }
  else if(waitingExitNotification && encoder->isRunning()) {
    DEBUG(fprintf(stderr," Encoder still running, sending KILL signal!\n"));
    if(recording) {
      recording = 0;
      card->stop();
    }
    encoder->kill();
  }
  else
    fprintf(stderr," OK \n");
}
